/********************************************************************
 *     Copyright (c) 2000 by Hewlett-Packard, All Rights Reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License, or (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *     GNU Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *     MA 02111-1307 USA
 *
 ********************************************************************/

package hp.infrared;

/**
 * This class implements an ultra packet which is broadcast in a
 * connectionless manner using the ultra protocol.
 * @see UltraSocket
 */

public final class UltraPacket
{

	private byte[] buffer;     
	private int buflen;        
	
        /**
      	 * Constructs an ultra packet for holding packets of length ilength.
      	 * @param ibuf the buffer
	 * @param ilength length of the buffer
	 */

	public UltraPacket(byte ibuf[], int ilength)
	{
		if (ibuf == null)
		{
			throw new NullPointerException();
		}
		if (ilength > ibuf.length)
		{
			throw new IllegalArgumentException("illegal length");
		}

		buffer = ibuf;
		buflen = ilength;
	}

	/**
 	 * Returns the data to be sent or that was received.
	 */

	public synchronized byte[] getData()
	{
		if (buffer == null)
			return null;
    
 		byte[] res = new byte[buffer.length];
		System.arraycopy(buffer, 0, res, 0, buffer.length);
    		return res;
	}

	/**
 	 * Returns the length of the data to be sent or that was received.
	 */
	public synchronized int getLength()
	{
		return buflen;
	}

	/**
	 * Sets the data
	 */

	public synchronized void setData(byte[] ibuf)
	{
		buffer = ibuf;
	}

	/** 
	 * Sets the length
	 */

	public synchronized void setLength(int ilength)
	{
		if (ilength > buffer.length)
		{
			throw new IllegalArgumentException("illegal length");
		}	
		buflen = ilength;
	}
}
