/********************************************************************
 *     Copyright (c) 2000 by Hewlett-Packard, All Rights Reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License, or (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *     GNU Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *     MA 02111-1307 USA
 *
 ********************************************************************/

package hp.infrared;

import java.io.FileDescriptor;
import java.io.IOException;
import java.net.SocketException;
import java.net.BindException;

class UltraSocketImpl
{
   
  private final static int MAX_PID = 127;
  private FileDescriptor fd;
  
  private native static FileDescriptor socketCreate()
    throws SocketException; 
    
  private native static void socketBind(FileDescriptor fd, int pid) throws SocketException;
      
  private native static void socketReceive(FileDescriptor fd,
					 UltraPacket pkt) throws IOException;

  private native static void socketSend(FileDescriptor fd, UltraPacket pkt)
    throws IOException;

  private native static void socketClose(FileDescriptor fd);
  
  protected void bind(int pid) throws SocketException
  {
    if (this.fd == null)
      throw new NullPointerException("socket FileDescriptor is null");
    
    if ((pid < 0) || (pid > MAX_PID))
      throw new IllegalArgumentException("PID out of range:" + pid);
    
    socketBind(this.fd, pid);
  }
  
  protected void close()
  {
      if (this.fd != null)
	socketClose(this.fd);
  }
  
  protected void create() throws SocketException
  {
    fd = socketCreate();
  }

  protected void receive(UltraPacket pkt) throws IOException
  {
    if (this.fd == null)
      throw new NullPointerException("socket FileDescriptor is null");
    
    if (pkt == null)
      throw new NullPointerException();
    
    socketReceive(this.fd,pkt);
  }
  
  protected void send(UltraPacket pkt) throws IOException
  {
    if (this.fd == null)
      throw new NullPointerException("socket FileDescriptor is null");
    
    if (pkt == null)
      throw new NullPointerException();
    
    socketSend(this.fd, pkt);
  }

}
