/********************************************************************
 *     Copyright (c) 2000 by Hewlett-Packard, All Rights Reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License, or (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *     GNU Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public 
 *     License along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *     MA 02111-1307 USA
 *
 ********************************************************************/

#include <string.h>
#include <sys/time.h>
#include <errno.h>
#include <sys/socket.h>

#include "hp_infrared_IrSocketInputStream.h"

/*
 * Class:     hp/infrared/IrSocketInputStream
 * Method:    nativeRead
 * Signature: (Ljava/io/FileDescriptor;[BII)I
 */
JNIEXPORT jint JNICALL Java_hp_infrared_IrSocketInputStream_nativeRead__Ljava_io_FileDescriptor_2_3BII
(JNIEnv *env, jobject obj, jobject fdobj, jbyteArray byteArray,
 jint off, jint len)
{
  int fd,res;
  jbyte *buffer;
  const char *errmsg;

  buffer = (*env)->GetByteArrayElements(env,byteArray, 0);

  fd = (*env)->GetIntField(env,fdobj,
                       (*env)->GetFieldID(env,(*env)->GetObjectClass(env,fdobj),
                                       "fd", "I"));

  if (fd != -1){
    res = recv(fd, buffer + off, len, 0);
    if ((res == -1) && (errno == 104)) {
      res = 0; // Ignore 'Connection reset by peer' - Dan
    }
    if (res == -1) errmsg = strerror(errno);
  }else{
    res = -1;
    errmsg = "Socket has already been closed.";
  }
  
  (*env)->ReleaseByteArrayElements(env,byteArray, buffer, 0);

  if (res == 0 && len != 0){
    res = -1;  // end of file reached
  }else if (res == -1)
    (*env)->ThrowNew(env,(*env)->FindClass(env,"java/io/IOException"), errmsg);

  return (jint) res;
}

/*
 * Class:     hp/infrared/IrSocketInputStream
 * Method:    nativeRead
 * Signature: (Ljava/io/FileDescriptor;)I
 */
JNIEXPORT jint JNICALL Java_hp_infrared_IrSocketInputStream_nativeRead__Ljava_io_FileDescriptor_2
(JNIEnv *env, jobject obj, jobject fdobj)
{
  int fd,res;
  unsigned char b;

  fd = (*env)->GetIntField(env,fdobj, (*env)->GetFieldID(env,(*env)->GetObjectClass(env,fdobj), "fd", "I"));

  res = recv(fd,&b, 1, 0);
  if ((res == -1) && (errno == 104)) {
    res = 0; // Ignore 'Connection reset by peer' - Dan
  }
  
  if (res == 0){
    res = -1;  // end of file reached
  }else if (res == -1){
    (*env)->ThrowNew(env,(*env)->FindClass(env,"java/io/IOException"), strerror(errno));
  }else{
    res = b;
  }
 
  return (jint) res;
}
