/********************************************************************
 *     Copyright (c) 2000 by Hewlett-Packard, All Rights Reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License, or (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *     GNU Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public 
 *     License along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *     MA 02111-1307 USA
 *
 ********************************************************************/

#include <string.h>
#include <errno.h>
#include <sys/socket.h>
#include "hp_infrared_IrSocketOutputStream.h"

/*
 * Class:     hp/infrared/IrSocketOutputStream
 * Method:    nativeWrite
 * Signature: (Ljava/io/FileDescriptor;[BII)V
 */
JNIEXPORT void JNICALL Java_hp_infrared_IrSocketOutputStream_nativeWrite__Ljava_io_FileDescriptor_2_3BII
(JNIEnv *env, jobject obj, jobject fdobj, jbyteArray b, jint off, jint len)
{
  int fd,ret;
  jbyte *data;
  int readLen = 0;
  const char* errmsg;

  data = (*env)->GetByteArrayElements(env,b, 0);

  fd = (*env)->GetIntField(env,fdobj,
			    (*env)->GetFieldID(env,(*env)->GetObjectClass(env,fdobj), "fd", "I"));

  if (fd != -1){
    while (readLen < len){
      ret = send(fd, data + off + readLen, len - readLen, 0);
      if (ret < 0) {
	if (errno == EINTR)
	  continue;
	else {
		errmsg = strerror(errno);
		break;
	}
      }
      else
	readLen +=ret;
    }
  }else{
    ret = -1;
    errmsg = "Socket has already been closed.";
  }

  if (ret == -1){
    (*env)->ThrowNew(env,(*env)->FindClass(env,"java/io/IOException"), errmsg);
  }
 
  (*env)->ReleaseByteArrayElements(env,b, data, 0);
}

/*
 * Class:     hp/infrared/IrSocketOutputStream
 * Method:    nativeWrite
 * Signature: (Ljava/io/FileDescriptor;I)V
 */
JNIEXPORT void JNICALL Java_hp_infrared_IrSocketOutputStream_nativeWrite__Ljava_io_FileDescriptor_2I
  (JNIEnv *env, jobject obj, jobject fdobj, jint b)
{
  int fd,ret;
  jbyte data = b;
  const char* errmsg;
  
  fd = (*env)->GetIntField(env,fdobj,
			    (*env)->GetFieldID(env,(*env)->GetObjectClass(env,fdobj), "fd", "I"));

  ret = send(fd, &data, 1, 0);

  if (ret == -1)
    (*env)->ThrowNew(env,(*env)->FindClass(env,"java/io/IOException"), strerror(errno));
}
