/********************************************************************
 *     Copyright (c) 2000 by Hewlett-Packard, All Rights Reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License, or (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *     GNU Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public 
 *     License along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *     MA 02111-1307 USA
 *
 ********************************************************************/

#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/time.h>
#include <stdio.h>
#include <sys/socket.h>
#include <unistd.h>
#include <stdio.h>
#include <linux/types.h>
#include <linux/irda.h>

#include "hp_infrared_UltraSocketImpl.h"

int fdValue(JNIEnv * env, jobject fdobj)
{
  int fd = -1;
  if (! fdobj)
    (*env)->ThrowNew(env,(*env)->FindClass(env,"java/lang/NullPointerException"), "Socket FileDescriptor is null");
  else
    fd = (*env)->GetIntField(env,fdobj, (*env)->GetFieldID(env,(*env)->GetObjectClass(env,fdobj), "fd", "I"));
  return fd;
}

/*
 * Class:     hp/infrared/UltraSocketImpl
 * Method:    socketCreate
 * Signature: ()Ljava/io/FileDescriptor;
 */
JNIEXPORT jobject JNICALL Java_hp_infrared_UltraSocketImpl_socketCreate
  (JNIEnv *env, jclass cls)
{
  jobject res = 0;
  int sfd = socket(AF_IRDA, SOCK_DGRAM, 1);
  
  if (sfd == -1)
    {
      (*env)->ThrowNew(env,(*env)->FindClass(env,"java/net/SocketException"),
		    strerror(errno));
    }
  else
    {
      jclass clazz;
      jfieldID id;
      clazz = (*env)->FindClass(env,"java/io/FileDescriptor");
      id = (*env)->GetFieldID(env,clazz, "fd", "I");
      res = (*env)->NewObject(env,clazz, (*env)->GetMethodID(env,clazz, "<init>", "()V"));
      (*env)->SetIntField(env,res, id, sfd);
    }
  
  return res;
}

/*
 * Class:     hp/infrared/UltraSocketImpl
 * Method:    socketBind
 * Signature: (Ljava/io/FileDescriptor;[BI)V
 */
JNIEXPORT void JNICALL Java_hp_infrared_UltraSocketImpl_socketBind
(JNIEnv *env, jclass cls, jobject fdobj, jint pid)
{
  int sfd = fdValue(env, fdobj);
  
  struct sockaddr_irda iaddr;
  
  iaddr.sir_family = AF_IRDA;
  iaddr.sir_lsap_sel  = pid;
  
  if (bind(sfd, (struct sockaddr*)&iaddr, sizeof iaddr) == -1)
    {
      (*env)->ThrowNew(env,(*env)->FindClass(env,"java/net/BindException"),
		    strerror(errno));
    }
  
}

/*
 * Class:     hp/infrared/UltraSocketImpl
 * Method:    socketReceive
 * Signature: (Ljava/io/FileDescriptor;Lhp/infrared/UltraPacket;)V
 */
JNIEXPORT void JNICALL Java_hp_infrared_UltraSocketImpl_socketReceive
(JNIEnv *env, jclass cls, jobject fdobj,  jobject pktobj)
{
  int bytes_to_receive, fromlen;
  jclass pktclazz;
  jfieldID pktId;
  jbyteArray jpktarr;
  jsize datalen;
  jbyte *jpkt;
  const char* errmsg;
  int bytes_received;

  int sfd = fdValue(env, fdobj);
  pktclazz = (*env)->GetObjectClass(env,pktobj);
  
  /************************************************************
   * get number of bytes to receive
   ************************************************************/
  bytes_to_receive = (*env)->GetIntField(env,pktobj, (*env)->GetFieldID(env,pktclazz, "buflen", "I"));
  if (bytes_to_receive < 0)
    {
      (*env)->ThrowNew(env,(*env)->FindClass(env,"java/lang/IllegalArgumentException"), "bytes to receive is < 0");
      return;
    }
  
  /************************************************************
  *
  * The ultra packet's buffer is filled with data received.
  *
  ************************************************************/
  pktId = (*env)->GetFieldID(env,pktclazz, "buffer", "[B");
  jpktarr = (jbyteArray) (*env)->GetObjectField(env,pktobj, pktId);
  
  if (! jpktarr)
    {
      (*env)->ThrowNew(env,(*env)->FindClass(env,"java/lang/NullPointerException"), "data buffer is null");
      return;
    }
  
  /************************************************************
   * check if there is enough space to receive bytes_to_receive
   ************************************************************/
  datalen = (*env)->GetArrayLength(env,jpktarr);
  if (bytes_to_receive > datalen)
    {
      bytes_to_receive = (int) datalen;
    }
  
  jpkt = (*env)->GetByteArrayElements(env,jpktarr, 0);

  /************************************************************
   * receive the packet
   ************************************************************/

  if (sfd != -1){
    // valid file descriptor, read data
    bytes_received = recv(sfd, jpkt, bytes_to_receive,0);
  }else{
    bytes_received = -1;
    errmsg = "Socket has already been closed.";
  }
  
  (*env)->ReleaseByteArrayElements(env,jpktarr, jpkt, 0);
  
  if (bytes_received == -1)
    {
      (*env)->ThrowNew(env,(*env)->FindClass(env,"java/io/IOException"), errmsg);
    }
  else if (bytes_received == -2)
    {
    (*env)->ThrowNew(env,(*env)->FindClass(env,"java/io/InterruptedIOException"), errmsg);
    }
}

/*
 * Class:     hp/infrared/UltraSocketImpl
 * Method:    socketSend
 * Signature: (Ljava/io/FileDescriptor;Lhp/infrared/UltraPacket;)V
 */
JNIEXPORT void JNICALL Java_hp_infrared_UltraSocketImpl_socketSend
  (JNIEnv *env, jclass cls, jobject fdobj, jobject pktobj)
{
  int bytes_tosnd;
  jclass pktclazz;
  jfieldID pktId;
  jbyteArray jpktarr;
  jsize datalen;
  jbyte *jpkt;
  int sfd,bytessent;

  sfd = fdValue(env, fdobj);
  
  pktclazz = (*env)->GetObjectClass(env,pktobj);
  
  /************************************************************
   * get number of bytes to send
   ************************************************************/
  bytes_tosnd = (*env)->GetIntField(env,pktobj, (*env)->GetFieldID(env,pktclazz,
							 "buflen", "I"));
  if (bytes_tosnd < 0)
    {
      (*env)->ThrowNew(env,(*env)->FindClass(env,"java/lang/IllegalArgumentException"), "packet length is < 0");
      return;
    }
  
  /************************************************************
   * get data packet to send
   ************************************************************/
  pktId = (*env)->GetFieldID(env,pktclazz, "buffer", "[B");
  jpktarr = (jbyteArray) (*env)->GetObjectField(env,pktobj, pktId);
  
  if (! jpktarr)
    {
      (*env)->ThrowNew(env,(*env)->FindClass(env,"java/lang/NullPointerException"), "data buffer is null");
      return;
    }

  /************************************************************
   * check if there is enough data to send bytes_tosnd
   * if not, then send only as much data as available
   ************************************************************/
  datalen = (*env)->GetArrayLength(env,jpktarr);
  if (bytes_tosnd > datalen)
    {
      bytes_tosnd = (int) datalen;
    }
  
  jpkt = (*env)->GetByteArrayElements(env,jpktarr, 0);
  
  /************************************************************
   * send the packet
   ************************************************************/
  bytessent = send(sfd, (char*)jpkt, bytes_tosnd, 0);
  (*env)->ReleaseByteArrayElements(env,jpktarr, jpkt, 0);
  
  if (bytessent == -1)
    {
      (*env)->ThrowNew(env,(*env)->FindClass(env,"java/io/IOException"), strerror(errno));
    }
  else
    {
      (*env)->SetIntField(env,pktobj, (*env)->GetFieldID(env,pktclazz, "buflen", "I"), bytessent);
    }
}

/*
 * Class:     hp/infrared/UltraSocketImpl
 * Method:    socketClose
 * Signature: (Ljava/io/FileDescriptor;)V
 */
JNIEXPORT void JNICALL Java_hp_infrared_UltraSocketImpl_socketClose
(JNIEnv *env, jclass cls, jobject fdobj)
{
  jclass clazz;
  jfieldID id;
  int sfd;

  if (! fdobj)
    {
      (*env)->ThrowNew(env,(*env)->FindClass(env,"hp/infrared/NullPointerException"), "FileDescriptor is null");
      return;
    }

  clazz = (*env)->GetObjectClass(env,fdobj);
  id = (*env)->GetFieldID(env,clazz, "fd", "I");
  
  sfd = (*env)->GetIntField(env,fdobj, id);
  
  if (sfd >= 0)
    {
      int res;
      // set an invalid fd
      (*env)->SetIntField(env,fdobj, id, (jint)-1);
      
      res = close(sfd);
    }

}

